"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const passport_1 = __importDefault(require("passport"));
const passport_google_oauth20_1 = __importDefault(require("passport-google-oauth20"));
const dotenv_1 = __importDefault(require("dotenv"));
const logger_1 = require("../utils/logger");
const db_1 = __importDefault(require("../db"));
const Users_1 = require("../db/schema/Users");
const drizzle_orm_1 = require("drizzle-orm");
dotenv_1.default.config();
const GoogleStrategy = passport_google_oauth20_1.default.Strategy;
const PROVIDER = "google";
const { GOOGLE_CLIENT_ID, GOOGLE_CLIENT_SECRET, GOOGLE_CALLBACK_URL } = process.env;
if (!GOOGLE_CLIENT_ID || !GOOGLE_CLIENT_SECRET || !GOOGLE_CALLBACK_URL) {
    logger_1.logger.error("Missing Google OAuth environment variables");
}
else {
    passport_1.default.use(new GoogleStrategy({
        clientID: GOOGLE_CLIENT_ID,
        clientSecret: GOOGLE_CLIENT_SECRET,
        callbackURL: GOOGLE_CALLBACK_URL,
        scope: ["email", "profile"],
        state: false,
    }, async (_accessToken, _refreshToken, profile, done) => {
        const fedCreds = await db_1.default
            .select({ user_id: Users_1.tableFederatedCredentials.user_id })
            .from(Users_1.tableFederatedCredentials)
            .where((0, drizzle_orm_1.and)((0, drizzle_orm_1.eq)(Users_1.tableFederatedCredentials.provider, PROVIDER), (0, drizzle_orm_1.eq)(Users_1.tableFederatedCredentials.subject, profile.id)));
        const exists = fedCreds.length > 0;
        if (exists) {
            const users = await db_1.default
                .select({
                firstName: Users_1.tableUsers.firstName,
                lastName: Users_1.tableUsers.lastName,
                email: Users_1.tableUsers.email,
                username: Users_1.tableUsers.username,
                role: Users_1.tableUsers.role,
            })
                .from(Users_1.tableUsers)
                .where((0, drizzle_orm_1.eq)(Users_1.tableUsers.id, fedCreds[0].user_id));
            if (!users[0])
                throw new Error("User not found");
            return done(null, users[0]);
        }
        else {
            try {
                const user = {
                    firstName: profile.name?.familyName,
                    lastName: profile.name?.givenName,
                    email: profile.emails?.[0].value ?? "",
                    username: profile.displayName,
                };
                const newUsers = await db_1.default
                    .insert(Users_1.tableUsers)
                    .values(user)
                    .onConflictDoNothing({ target: Users_1.tableUsers.id })
                    .returning();
                if (!newUsers)
                    throw new Error("Failed to create user");
                const { id } = newUsers[0];
                const fedCreds = {
                    user_id: id,
                    provider: PROVIDER,
                    subject: profile.id,
                };
                const newFedCred = await db_1.default
                    .insert(Users_1.tableFederatedCredentials)
                    .values(fedCreds)
                    .onConflictDoNothing({
                    target: Users_1.tableFederatedCredentials.user_id,
                })
                    .returning();
                if (!newFedCred)
                    throw new Error("Failed to create new federated credentials");
                return done(null, {
                    firstName: newUsers[0].firstName,
                    lastName: newUsers[0].lastName,
                    email: newUsers[0].email,
                    username: newUsers[0].username,
                    role: newUsers[0].role,
                });
            }
            catch (error) {
                logger_1.logger.error("Failed to create user");
            }
        }
    }));
}
exports.default = passport_1.default;
