"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const express_1 = __importDefault(require("express"));
const users_1 = __importDefault(require("./routes/users"));
const auth_1 = __importDefault(require("./routes/auth"));
const db_1 = require("./middleware/db");
const logger_1 = require("./utils/logger");
const helmet_1 = __importDefault(require("helmet"));
const swagger_1 = __importDefault(require("../swagger"));
const dotenv_1 = __importDefault(require("dotenv"));
dotenv_1.default.config();
const port = process.env.LOCAL_API_PORT;
const app = (0, express_1.default)();
// Middleware
app.disable("x-powered-by");
app.use((0, helmet_1.default)());
app.use(express_1.default.json());
app.use(db_1.attachDb);
// Routers
app.use("/users", users_1.default);
app.use("/auth", auth_1.default);
// Run
app.listen(port, () => {
    logger_1.logger.info(`Running: http://localhost:${port}`);
    logger_1.logger.info(`Swagger API docs are available at http://localhost:${port}/api-docs`);
});
(0, swagger_1.default)(app);
